using NAT.Common.Storables;
using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace NAT.Common.Components
{
    abstract class UIModManagerBase
    {
        MonoBehaviour _parent;
        protected Transform parentUI;
        protected readonly List<UIMod> uiMods = new List<UIMod>();
        protected bool initialized;
        protected StorableBool skinMaterials2ModifiedJsb;

        internal JSONStorableAction SwitchToOtherPluginAction { get; set; }
        internal UIDynamicSlider DrySkinFrictionSlider { get; set; }

        protected UIMod NewUIMod(
            MonoBehaviour parent,
            string uiModId,
            string targetName,
            Func<UIMod, IEnumerator> changesFunc,
            Func<bool> checkPluginEnabled,
            bool useDisabledStyle = false
        )
        {
            _parent = parent;
            var uiMod = new UIMod(_parent, uiModId, parentUI, targetName, changesFunc)
            {
                CheckPluginEnabled = checkPluginEnabled,
                UseDisabledStyle = useDisabledStyle,
            };
            uiMods.Add(uiMod);
            return uiMod;
        }

        internal virtual void SetFrictionUIVisibility()
        {
        }

        protected void ShowFrictionUI(bool frictionHandlerEnabled)
        {
            if(!_parent.enabled)
            {
                return;
            }

            if(frictionHandlerEnabled)
            {
                uiMods.Find(uiMod => uiMod.Id == Constant.SKIN_MATERIALS_2_FRICTION).Enable();
            }

            uiMods.Find(uiMod => uiMod.Id == Constant.SKIN_MATERIALS_2_ADD_BTN).Disable();

            skinMaterials2ModifiedJsb.val = true;
        }

        protected void HideFrictionUI(bool frictionHandlerEnabled)
        {
            if(!_parent.enabled)
            {
                return;
            }

            uiMods.Find(uiMod => uiMod.Id == Constant.SKIN_MATERIALS_2_FRICTION).Disable();
            if(frictionHandlerEnabled)
            {
                uiMods.Find(uiMod => uiMod.Id == Constant.SKIN_MATERIALS_2_ADD_BTN).Enable();
            }

            skinMaterials2ModifiedJsb.val = false;
        }

        internal virtual void Disable()
        {
            foreach(var uiMod in uiMods)
            {
                uiMod.Disable();
            }
        }

        internal abstract void Enable();

        internal void Destroy()
        {
            foreach(var uiMod in uiMods)
            {
                uiMod.Destroy();
            }
        }
    }
}
